<?php
/*
Plugin Name: ショートコードでカスタムHTML出力
Plugin URI: https://pera-lab.com/archives/947
Description: 投稿ページにカスタムHTMLを入力できるテキストエリアを用意し、ショートコードによって記事中の任意の位置で出力できます。（主に広告貼付けを目的としたものです）(ShortCodePrintCustomHTML)
Version: 1.1
Author: pera-lab
Author URI: https://pera-lab.com/
License: GPL2

<ショートコードでカスタムHTML出力> is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.
 
<ショートコードでカスタムHTML出力> is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with <ショートコードでカスタムHTML出力>. If not, see <http://www.gnu.org/licenses>.


更新履歴

Ver1.1	2018/03/07
予約投稿の時、カスタムフィールドの値が消えてしまうことを対処。
nonce処理を導入。
パーミッション確認を行うようにした。
自動保存の場合は、処理をしないようにした。
名前ラベルにindex番号を付けるようにした。

*/


//取り扱う最大アイテム数
const SCPCH_ITEM_MAX = 10;


class ScpchItemData{
	public $index = PHP_INT_MAX;
	public $name = '';
	public $html = '';
	public $centering = '';
	
	function __construct($index){
		$this->index = $index;
	}
	
	function TagID_Name(){
		return "scpch-name-forid-{$this->index}";
	}
	function TagName_Name(){
		return "scpch-name-nameid-{$this->index}";
	}

	function TagID_Centering(){
		return "scpch-centering-forid-{$this->index}";
	}
	function TagName_Centering(){
		return "scpch-centering-nameid-{$this->index}";
	}

	function TagID_Html(){
		return "scpch-html-forid-{$this->index}";
	}
	function TagName_Html(){
		return "scpch-html-nameid-{$this->index}";
	}
	
	function FormMake(){
		?>
		<div>
		
		<!-- 名前 -->
		<div>
		<label for="<?php echo $this->TagID_Name(); ?>">名前(<?php echo $this->index; ?>)</label>
		<input id="<?php echo $this->TagID_Name(); ?>" name="<?php echo $this->TagName_Name(); ?>" type="text" size="30" value="<?php echo esc_attr($this->name); ?>">
		<!-- 中央寄せ -->
		<label for="<?php echo $this->TagID_Centering(); ?>" >　中央寄せ</label>
		<input id="<?php echo $this->TagID_Centering(); ?>" name="<?php echo $this->TagName_Centering(); ?>" type="checkbox" value="checked" <?php echo $this->centering; ?> >
		</div>

		<!-- カスタムHTML -->
		<div>
		<p><label for="<?php echo $this->TagID_Html(); ?>">カスタムHTML(広告コード)</label></p>
		<textarea id="<?php echo $this->TagID_Html(); ?>" name="<?php echo $this->TagName_Html(); ?>" rows="7" cols="50"><?php echo esc_html($this->html) ?></textarea>
		</div>
		<?php echo $this->html; ?>
		
		</div>
		<?php
	}
}

//--------------------------------------------------------------
//	アクションフックに登録：管理画面にカスタムボックスをエントリー
//--------------------------------------------------------------
add_action(
	'add_meta_boxes', 
	function(){
		$screens = array('post', 'page');	//投稿ページと固定ページに表示
		foreach($screens as $scrn){
			add_meta_box('peralab-scpch-custombox', 'ショートコードでカスタムHTML出力', 
				'PeralabScpch_CustomBoxCreate', $scrn, 'normal', 'default', null);
		}
	}
);

//--------------------------------------------------------------
//	カスタムボックスを作成
//--------------------------------------------------------------
function PeralabScpch_CustomBoxCreate($post){
	//nonce作成
	wp_nonce_field('action-noncekey-peralab-scpch-input', 'noncename-scpch-metafield');
	
	for($i = 0; $i < SCPCH_ITEM_MAX; $i++){
		$data = get_post_meta($post->ID, "scpch_item_list{$i}", true);
		if(isset($data) == false || $data == ''){
			$data = new ScpchItemData($i);
		}
		
		$data->FormMake();
		if($i < SCPCH_ITEM_MAX-1){
			echo '<hr>';
		}
	}
}

//--------------------------------------------------------------
//	カスタムボックス内のフィールド値更新処理
//--------------------------------------------------------------
add_action(
	'save_post', 
	function($post_id){
		//nonceを確認
		if(isset($_POST['noncename-scpch-metafield']) == false 
				|| wp_verify_nonce($_POST['noncename-scpch-metafield'], 'action-noncekey-peralab-scpch-input') == false) {
			return;	//nonceを認証できなかった
		}
		
		//自動保存ルーチンかどうかチェック。そうだった場合はフォームを送信しない(何もしない)
		if(defined('DOING_AUTOSAVE') && DOING_AUTOSAVE){
			return;
		}
		
		//パーミッション確認
		if(isset($_POST['post_type'])){
			if($_POST['post_type'] == 'page'){
				if(!current_user_can('edit_page', $post_id)){
					return;	//固定ページを編集する権限がない
				}
			}
			else{
				if(!current_user_can('edit_post', $post_id)){
					return;	//記事を編集する権限がない
				}
			}
		}
		
		//== 確認ここまで ==
		

		for($i = 0; $i < SCPCH_ITEM_MAX; $i++){
			$post_data = new ScpchItemData($i);
			$update_count = 0;
			
			if(isset($_POST[$post_data->TagName_Name()])){
				$post_data->name = $_POST[$post_data->TagName_Name()];
				$update_count++;
			}
			if(isset($_POST[$post_data->TagName_Centering()])){
				$post_data->centering = $_POST[$post_data->TagName_Centering()];
				$update_count++;
			}
			if(isset($_POST[$post_data->TagName_Html()])){
				$post_data->html = $_POST[$post_data->TagName_Html()];
				$update_count++;
			}
			
			//予約投稿の時、データが有るにも関わらず$_POSTが取得できないため、取得できている時のみ更新処理を行う。
			//issetは''の場合もデータ有りと判定してくれるので、任意の削除はupdate_countがプラスされているはず
			if($update_count > 0){
				update_post_meta($post_id, "scpch_item_list{$i}", $post_data);
			}
		}
	}
);


//==============================================================================
//
//	ショートコード
//
//==============================================================================
add_shortcode(
	'print_html', 
	function($atts){
		$atts = shortcode_atts(array(
					'name' => '',
				), 
				$atts,
				'print_html'//ショートコード名。省略可能だが含めるのが推奨
			);
		extract($atts);
		
		if($name == ''){
			return '<p style="color:red;">print_htmlのnameが設定されていません<br>[print_html name=\'名前\']と記述してください</p>';
		}
		
		for($i = 0; $i < SCPCH_ITEM_MAX; $i++){
			$data = get_post_meta(get_the_ID(), "scpch_item_list{$i}", true);
			if(isset($data) == false || $data == ''){
				continue;
			}
			
			if($data->name == $name){
				if($data->centering == "checked"){
					return '<div style="text-align: center;">' . $data->html . '</div>';
				}
				return $data->html;
			}
		}
		
		return '<p style="color:red;">print_htmlのname（' . $name . '）が見当たりません</p>';
	}
);


?>
